"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RCONClient = void 0;
const node_events_1 = require("node:events");
const node_net_1 = require("node:net");
const node_process_1 = require("node:process");
const PacketBuilder_1 = require("./protocol/PacketBuilder");
const PacketHandler_1 = require("./protocol/PacketHandler");
const PacketType_1 = require("./util/PacketType");
/**
 * RCONClient Class
 * @see https://wiki.vg/RCON for more information
 */
class RCONClient extends node_events_1.EventEmitter {
    /**
     * Instantiate a new RCONClient instance
     * @param host Host for this RCONClient
     * @param password Password used to login
     * @param port Port to use, defaults to 25575
     */
    constructor(host, password, port = 25575) {
        super();
        this.host = host;
        this.port = port;
        this.authenticated = false;
        this.password = password;
        this.handler = new PacketHandler_1.RCONPacketHandler();
    }
    /**
     * Connect to the Minecraft server
     * @returns The RCONClient
     */
    connect() {
        this.socket = new node_net_1.Socket();
        this.socket.on('connect', () => {
            this.connected = true;
            this.emit('connect');
            const { buffer, requestId } = new PacketBuilder_1.RCONPacketBuilder(PacketType_1.RCONPacketType.LOGIN, this.password);
            this.authRequestId = requestId;
            this.socket.write(buffer);
        });
        this.socket.on('data', (buffer) => this.handleMessage(buffer));
        this.socket.on('end', () => this.disconnect());
        this.socket.on('close', () => this.disconnect());
        this.socket.on('error', (error) => {
            this.disconnect();
            this.emit('error', error, false);
        });
        this.socket.connect({
            host: this.host,
            port: this.port,
        });
        return this;
    }
    /**
     * Execute a command on the server
     * @param command Command to execute on the server
     * @returns The request ID (use it to catch the response)
     */
    executeCommand(command) {
        if (!this.authenticated)
            throw new Error("Can't send command before authentication!");
        const { buffer, requestId } = new PacketBuilder_1.RCONPacketBuilder(PacketType_1.RCONPacketType.COMMAND, command);
        this.socket.write(buffer);
        return requestId;
    }
    /**
     * Disconnect the client and end the connection
     */
    disconnect() {
        this.connected = false;
        this.socket.end();
        this.emit('disconnect');
    }
    handleMessage(buffer) {
        this.emit('raw_message', buffer);
        const { requestId, packet } = this.handler.handle(buffer);
        switch (packet.type) {
            case PacketType_1.RCONPacketType.RESPONSE:
                this.emit('response', packet);
                break;
            case PacketType_1.RCONPacketType.ERROR:
                if (requestId === this.authRequestId) {
                    const emitted = this.emit('authentication_failed');
                    if (!emitted)
                        throw new Error('Authentication failed, please handle the `authentication_failed` event');
                }
                const emitted = this.emit('error', new Error(packet.payload), true);
                if (!emitted)
                    throw new Error(packet.payload);
                break;
            case PacketType_1.RCONPacketType.COMMAND:
                // For some reason, when authenticating
                // a Command packet is sent when successful
                if (requestId === this.authRequestId) {
                    this.authenticated = true;
                    this.emit('authenticated');
                }
                break;
            default:
                (0, node_process_1.emitWarning)(`Unknown RCONPacketType (type=${packet.type})`);
                break;
        }
    }
}
exports.RCONClient = RCONClient;
